// ME4SE - A MicroEdition Emulation for J2SE 
//
// Copyright (C) 2001 Stefan Haustein, Oberhausen (Rhld.), Germany
//
// Contributors:
//
// STATUS: API complete, optional image functionality missing
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version. This program is
// distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
// License for more details. You should have received a copy of the
// GNU General Public License along with this program; if not, write
// to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

package javax.microedition.lcdui;

import javax.microedition.midlet.*;
import java.util.Vector;
import org.me4se.impl.lcdui.*;

public class ChoiceGroup extends Item implements Choice {

    int type;
    Vector group;

    public ChoiceGroup(String label, int type) {
        super(label);
        this.type = type;
        if (type == EXCLUSIVE)
            group = new Vector();
    }

    public ChoiceGroup(
        String label,
        int type,
        String[] stringItems,
        Image[] imageItems) {
        this(label, type);
        for (int i = 0; i < stringItems.length; i++)
            append(stringItems[i], imageItems == null ? null : imageItems[i]);
    }

    /** internal */

    DeviceLabel getLabel(int i) {
        return (DeviceLabel) lines.elementAt(i + 1);
    }

    public int append(String stringItem, Image imageItem) {
        insert(size(), stringItem, imageItem);
        return size() - 1;
    }

    public void delete(int index) {

        int i = delete();

        if (group != null)
            group.removeElementAt(index);

        lines.removeElementAt(index + 1);

        if (i != -1)
            form.insert(i, this);

        readd(i);
    }

    public Image getImage(int index) {
        return (Image) getLabel(index).object;
    }

    public int getSelectedFlags(boolean[] flags) {
        int count = 0;
        for (int i = 0; i < size(); i++) {
            flags[i] = getLabel(i).selected();
            if (flags[i]) count++;
        }
        
        for (int i = size(); i < flags.length; i++)
            flags[i] = false;

        return count;
    }

    public int getSelectedIndex() {
        if (type != MULTIPLE)
            for (int i = 0; i < size(); i++)
                if (getLabel(i).selected())
                    return i;

        return -1;
    }

    public String getString(int index) {
        return getLabel(index).getText();
    }

    public void insert(int index, String stringItem, Image imageItem) {

        int i = delete();

        DeviceLabel cb = new DeviceLabel("item", true);
        cb.selectButtonRequired = true;
        cb.checkbox = true;
        if (imageItem != null) {
            cb.image = imageItem._image;
            cb.object = imageItem;
        }
        cb.setText(stringItem);
        cb.highlight = true;
        if (group != null) {
            group.insertElementAt(cb, index);
            cb.group = group;
            if (group.size() == 1)
                cb.select(true);
        }
        lines.insertElementAt(cb, index + 1);
        cb.invalidate();

        readd(i);
    }

    public boolean isSelected(int index) {
        return getLabel(index).selected();
    }

    public void set(int index, String str, Image img) {
        boolean sel = isSelected(index);
        delete(index);
        insert(index, str, img);
        setSelectedIndex(index, sel);
    }

    public void setLabel(int index, String s) {
        getLabel(index).setText(s);
    }

    public void setSelectedFlags(boolean[] flags) {

        for (int i = 0; i < size(); i++) {
            getLabel(i).select(flags[i]);
            if (flags[i] && group != null)
                return;
        }

        if (group != null && size() > 0)
            setSelectedIndex(0, true);
    }

    public void setSelectedIndex(int index, boolean state) {
        getLabel(index).select(state);
    }

    public int size() {
        return lines.size() - 1;
    }

}
