// ME4SE - A MicroEdition Emulation for J2SE 
//
// Copyright (C) 2001 Stefan Haustein, Oberhausen (Rhld.), Germany
//
// Contributors:
//
// STATUS: Dummy class with complete API
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version. This program is
// distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
// License for more details. You should have received a copy of the
// GNU General Public License along with this program; if not, write
// to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

package javax.microedition.lcdui;

import org.me4se.impl.lcdui.*;
import org.kobjects.scm.*;
import java.util.*;

public class DateField extends Item {

    public static final int DATE = 1;
    public static final int TIME = 2;
    public static final int DATE_TIME = 3;

    int mode;
    TimeZone timeZone;
    //    Date date = new Date();
    TextComponent dateField = new TextComponent("dateField");
    TextComponent timeField = new TextComponent("dateField");

    public DateField(String label, int mode, TimeZone timezone) {
        super(label);
        this.timeZone = timeZone;

	setDate (null);

	if ((mode & DATE) != 0)
	    lines.addElement(dateField);

	if ((mode & TIME) != 0)
	    lines.addElement(timeField);

        this.mode = mode;
    }

    public DateField(String label, int mode) {
        this(label, mode, TimeZone.getDefault());
    }

    public Date getDate() {
	try {
	    Calendar c = Calendar.getInstance (timeZone);
	    
	    if ((mode & DATE) != 0) { 
		String s = dateField.getText ();
		c.set(c.YEAR, Integer.parseInt (s.substring (0,4)));
		c.set(c.MONTH, Integer.parseInt (s.substring (5, 7)));
		c.set(c.DAY_OF_MONTH, Integer.parseInt (s.substring (8,10)));
	    }
	    else 	    
		c.setTime (new Date(0));

	    if ((mode & TIME) != 0) {
		String s = timeField.getText();
		c.set(c.HOUR_OF_DAY, Integer.parseInt (s.substring (0, 2)));
		c.set(c.MINUTE, Integer.parseInt (s.substring (3,5)));
	    }
	    else {
		c.set(c.HOUR_OF_DAY, 0);
		c.set(c.MINUTE, 0);
	    }
	    c.set(c.SECOND, 0);
	    c.set(c.MILLISECOND, 0);

	    return c.getTime ();
	}
	catch (Exception e) {
	    return null;
	}
    }

    String digits (int i, int n) {
	String result = ""+ i;
	while (result.length () < n) 
	    result = "0"+result;
	return result;
    }

    public int getInputMode() {
        return mode;
    }

    public void setInputMode(int mode) {
        this.mode = mode;
    }

    public void setDate(Date date) {

	if (date == null) {
	    dateField.setText("yyyy-mm-dd");
	    timeField.setText("hh:mm");
	}
	else {
	    Calendar c = Calendar.getInstance (timeZone);
	    c.setTime (date);
	    dateField.setText (digits (c.get(c.YEAR), 4) 
			       + "-" + digits (c.get(c.MONTH)+1, 2) 
			       + "-" + digits (c.get(c.DAY_OF_MONTH)+1, 2));

	    timeField.setText (digits (c.get(c.HOUR_OF_DAY), 2) 
			       + ":" + digits (c.get(c.MINUTE), 2));
	}
    }
}
