// ME4SE - A MicroEdition Emulation for J2SE 
//
// Copyright (C) 2001 Stefan Haustein, Oberhausen (Rhld.), Germany
//
// Contributors:
//
// STATUS: API complete
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version. This program is
// distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
// License for more details. You should have received a copy of the
// GNU General Public License along with this program; if not, write
// to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

package javax.microedition.lcdui;

import java.util.*;
import java.awt.event.*;
import javax.microedition.midlet.*;
import org.me4se.impl.lcdui.*;
import org.kobjects.scm.*;


/** manages commands, command listeners */

public abstract class Displayable {

    CommandListener commandListener;
    java.util.Vector commands = new Vector();

    Display display;
    DisplayableContainer container;


 //   set to true if soft buttons shall not be shown on a Canvas 
  //  boolean hideSoftButtons;

    /** set to true if the device does not provide a select button */
    boolean noSelectButton;

    /** set to true if the current form or *item* requires a select button */
    boolean selectButtonRequired;




    Displayable() {

        Display.check();

         noSelectButton =
            ApplicationManager.manager.skin != null
                && ApplicationManager.manager.properties.get("button.SELECT")
                    == null;
        
    }


    /**
     * notify the Displayable that it is displayed on the screen now
     */

    void _showNotify () {   
    }

 



    //------------- public method -----------------------

    public synchronized void addCommand(Command cmd) {

        if (commands.indexOf(cmd) != -1)
            return;
        for (int i = 0; i < commands.size(); i++) {
            Command curCmd = (Command) commands.elementAt(i);
            if (cmd.getPriority() < curCmd.getPriority()) {
                commands.insertElementAt(cmd, i);
                container.updateButtons();
                return;
            }
        }
        commands.insertElementAt(cmd, commands.size());
        container.updateButtons();
    }

   
       void handleCommand(Command cmd) {

        if (cmd == container.MENU_COMMAND) 
            display.setCurrent(new CommandList(this));
        else if (cmd == container.SELECT_COMMAND) {
            //System.out.println ("Select observed");
            ScmComponent f = container.getFocusOwner();
            if (f instanceof DeviceLabel) {
                //System.out.println ("action performed");
                 ((DeviceLabel) f).action();
            }
        }
        else if (cmd != null && commandListener != null)
            commandListener.commandAction(cmd, this);

    }
   
   
//    void notifyDisplayed () {
 //   }
 
    public boolean isShown() {
        return display != null && display.current == this;
    }

    public void setCommandListener(CommandListener commandListener) {
        this.commandListener = commandListener;
    }

/*    void setFocus() {
    }*/

    public void removeCommand(Command cmd) {
        int idx = commands.indexOf(cmd);
        if (idx == -1)
            return;
        commands.removeElementAt(idx);

        container.updateButtons();
    }
  
}

