// ME4SE - A MicroEdition Emulation for J2SE 
//
// Copyright (C) 2001 Stefan Haustein, Oberhausen (Rhld.), Germany
//
// Contributors:
//
// STATUS: API Complete
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version. This program is
// distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
// License for more details. You should have received a copy of the
// GNU General Public License along with this program; if not, write
// to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

package javax.microedition.lcdui;

import java.util.*;
import org.me4se.impl.lcdui.*;
import java.awt.event.*;
import org.kobjects.scm.*;

public class Form extends Screen {

    Vector items = new Vector();
    ItemStateListener itemStateListener;

    ScmScrollPane scrollPane = new ScmScrollPane ();
    DeviceList list = new DeviceList(this);

/*    
 * this should be moved to the corresponding item... 
 * 
 * FocusListener focusListener = new FocusAdapter() {
        public void focusGained(FocusEvent ev) {
            boolean sbr =
                (ev.getSource() instanceof DeviceLabel)
                    && ((DeviceLabel) ev.getSource()).highlight;

            if (sbr != selectButtonRequired) {
                selectButtonRequired = sbr;
                container.updateButtons();
            }
        }
    };*/

   void _showNotify () {
        if (size () == 0) return;
        list.validateFocus ();
   }

    public Form(String title) {
        super(title);
        scrollPane.add(list);
	ScmScrollBar sb = new ScmScrollBar ();
	sb.setBackground (javax.microedition.midlet.ApplicationManager.manager.bgColor);
	scrollPane.setVerticalBar (sb);
        container.setMain (scrollPane);
    }

    public Form(String title, Item[] items) {
        this(title);
        for (int i = 0; i < items.length; i++)
            append(items[i]);
    }

    int getComponentIndex(int index) {
        int ci = 0; 
        for (int i = 0; i < index; i++)
            ci += ((Item) items.elementAt(i)).lines.size();
        return ci;
    }

    void insert(int index, Item item) {
        if (index > size() || index < 0)
            throw new IndexOutOfBoundsException();

        if (item.form != null)
            throw new IllegalStateException();
        item.form = this;

        int cindex = getComponentIndex(index);

        for (int i = 0; i < item.lines.size(); i++) {
            ScmComponent c = (ScmComponent) item.lines.elementAt(i);
            list.add(c, cindex++);
      //      c.setVisible(true);
      //      c.addFocusListener(focusListener);
      //      c.addKeyListener(eventHandler);

            //System.out.println ("form.add: "+c);

            c.invalidate();
        }

        items.insertElementAt(item, index);
    }

    public int append(Item item) {
        int i = items.size();
        insert(i, item);
        return i;
    }

    public int append(String s) {
        return append(new StringItem(null, s));
    }

    public int append(Image img) {
        return append(new ImageItem(null, img, 0, null));
    }

    public void delete(int index) {
        if (index >= size() || index < 0)
            throw new IndexOutOfBoundsException();

        int ci = getComponentIndex(index);
        Item item = (Item) items.elementAt(index);
        for (int i = 0; i < item.lines.size(); i++) {
            ScmComponent c = list.getComponent(ci);
           // c.setVisible(false);
           // c.removeFocusListener(focusListener);
           // c.removeKeyListener(eventHandler);
            list.remove(ci);
        }
        item.form = null;
        items.removeElementAt(index);
    }

    /** internal method, used when rebuilding items with
    a different set of lines... */

    int delete(Item item) {
        int i = items.indexOf(item);
        if (i != -1)
            delete(i);
        return i;
    }

    public Item get(int index) {
        if (index >= size() || index < 0)
            throw new IndexOutOfBoundsException();
        return (Item) items.elementAt(index);
    }

    public void set(int index, Item item) {
        delete(index);
        insert(index, item);
    }

    public int size() {
        return items.size();
    }

    /*    void validate () {
    	bar.setMaximum (itemPanel.getMinimumSize ().height);
    
    bar.setVisibleAmount 
        (scroll.getSize ().height 
         - scroll.getInsets ().top 
         - scroll.getInsets ().bottom);
    
    bar.setVisible (bar.getMaximum () > bar.getVisibleAmount ());
    
    itemPanel.invalidate ();
    panel.validate ();
    }
    */

    public void setItemStateListener(ItemStateListener iListener) {

        itemStateListener = iListener;
    }

    private void notifyItemChanged(Item item) {
        itemStateListener.itemStateChanged(item);
    }


  public String toString () {
        StringBuffer buf = new StringBuffer (getClass() + "(lcduiList)[");
        for (int i = 0; i < size (); i++) {
            if (i > 0) buf.append (',');
            buf.append (get(i).toString());
        }
        buf.append(']');
        return buf.toString ();
    }

   /* void setFocus() {
        list.setFocus();
    }*/
}
