// ME4SE - A MicroEdition Emulation for J2SE 
//
// Copyright (C) 2001 Stefan Haustein, Oberhausen (Rhld.), Germany
//
// Contributors: Geoff Hubbard, Andre Gerard
//
// STATUS: 
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version. This program is
// distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
// License for more details. You should have received a copy of the
// GNU General Public License along with this program; if not, write
// to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

package javax.microedition.lcdui;

import java.io.*;
import java.awt.image.*;
import java.util.Vector;

import javax.microedition.midlet.*;

public class Image {

    class Color2GrayFilter extends RGBImageFilter {
        public int filterRGB(int x, int y, int rgb) {
            //return rgb & 0x0ffffff;
            return ApplicationManager.manager.getDeviceColor(rgb, false);
        }
    }

    /*
    class BitmapImageProducer implements ImageProducer {
    // convert data from siemens to a reasonable format,
    // then produce the whole image at once?
    }
    */

    public boolean _transparent;
    public java.awt.Image _image;

    boolean mutable;
    static java.awt.Component helper = new java.awt.Panel();
    String name;

    // shouldn we make a copy here?
    Image(java.awt.Image image, boolean mutable) {
        this._image = image;
        this.mutable = mutable;
    }

    Image(InputStream is) {
        Display.check();

        Color2GrayFilter filter = new Color2GrayFilter();

        try {
            java.awt.Image im = ApplicationManager.getPngImage(is);
            _image =
                java.awt.Toolkit.getDefaultToolkit().createImage(
                    new FilteredImageSource(im.getSource(), filter));
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static Image createImage(int w, int h) {
        Display.check();
        return new Image(
            ApplicationManager.manager.awtContainer.createImage(w, h),
            true);
    }

    public static Image createImage(byte[] data, int start, int len) {
        Display.check();
        return new Image(new ByteArrayInputStream(data, start, len));
    }

    public static Image createImage(String name) throws IOException {
        Display.check();

        name = name.replace('\\', '/');
        if (!name.startsWith("/"))
            name = "/" + name;

        InputStream is;

        if (ApplicationManager.manager.classLoader != null) {
            is =
                ApplicationManager.manager.classLoader.getResourceAsStream(
                    name);
        }
        else {
            is = Image.class.getResourceAsStream(name);
        }

        if (is == null)
            throw new IOException("Resource '" + name + "' not found!");

        Image img = new Image(is);
        img.name = name;
        return img;
    }

    /** additional siemes method */

    public static Image createImageFromBitmap(
        byte[] bytes,
        int width,
        int height) {
        //	System.out.println( 
        // "createImageFromBitmap width=" + width + " height=" + height );
        Display.check();

        int pix[] = new int[width * height];
        int index = 0;

        // if( width%8 > 0 )  handled by "+7" above  
        //	 numOfBytes++;

        int srcPos = 0;
        int dstPos = 0;

        int bgcolor = ApplicationManager.manager.bgColor.getRGB();

        for (int y = 0; y < height; y++) {
            int mask = 128;
            for (int x = 0; x < width; x++) {
                pix[dstPos++] =
                    0x0ff000000
                        | ((((bytes[srcPos] & mask) == 0) ? bgcolor : 0));

                //	System.out.print (pix [dstPos-1] == 0x0ff000000  ? "_" : "X");

                mask = mask >> 1;
                if (mask == 0) {
                    mask = 128;
                    srcPos++;
                }
            }

            if (mask != 128)
                srcPos++;
            //System.out.println ();
        }

        return new Image(
            java.awt.Toolkit.getDefaultToolkit().createImage(
                new MemoryImageSource(width, height, pix, 0, width)),
            true);
    }

    public static Image createTransparentImageFromBitmap(
        byte[] bytes,
        int width,
        int height) {

        Display.check();

        //System.out.println( 
        // "createTransparentImageFromBitmap width=" + width + " height=" + height );

        int pix[] = new int[width * height];
        int index = 0;

        // if( width%8 > 0 )  handled by "+7" above  
        //	 numOfBytes++;

        int srcPos = 0;
        int dstPos = 0;

        int[] colors =
            {
                ApplicationManager.manager.bgColor.getRGB() & 0x0ffffff,
                0x0ff000000 | ApplicationManager.manager.bgColor.getRGB(),
                0x0ff000000,
                0x0ff000000 };

        for (int y = 0; y < height; y++) {
            int shift = 6;
            for (int x = 0; x < width; x++) {
                pix[dstPos++] = colors[(bytes[srcPos] >> shift) & 3];

                //	System.out.print ((bytes [srcPos] >> shift) & 3);

                shift -= 2;
                if (shift < 0) {
                    shift = 6;
                    srcPos++;
                }
            }
            if (shift != 6)
                srcPos++;
            //System.out.println ();
        }

        Image ret =
            new Image(
                java.awt.Toolkit.getDefaultToolkit().createImage(
                    new MemoryImageSource(width, height, pix, 0, width)),
                true);
        ret._transparent = true;
        return ret;
    }

    public static Image createImage(Image source) {
        Display.check();

        if (source == null)
            throw new NullPointerException();
        if (!source.isMutable())
            return source;

        Image copy = createImage(source.getWidth(), source.getHeight());
        copy.getGraphics().drawImage(
            source,
            0,
            0,
            Graphics.TOP | Graphics.LEFT);

        copy.mutable = false;
        return copy;
    }

    public Graphics getGraphics() {
        if (!mutable)
            throw new IllegalStateException();
        return new Graphics(null, _image.getGraphics());
    }

    public int getWidth() {
        return _image.getWidth(null);
    }

    public int getHeight() {
        return _image.getHeight(null);
    }

    public boolean isMutable() {
        return mutable;
    }
}
