/*
 * $Id:$
 */

/**
 * @author Michael Kroll, michael.kroll@trantor.de
 */

package org.me4se.impl.gcf;

import javax.microedition.io.*;
import javax.microedition.midlet.*;
import java.io.*;
import java.net.*;
import java.util.*;

/*
 * Limitations:
 * 
 * 
 * 
 */

public class ConnectionImpl_file extends ConnectionImpl implements FileConnection {

	File connection = null;
	int mode = -1;
	String url = null;

	public void initialise(Properties properties) {

	}

	public void open(String url, int mode, boolean timeouts) throws IOException {

		this.mode = mode;
		url = url.substring(8);
		this.url = url;
		connection = new File(url);
	}

	/*
	 * Will be left out !
	 */
	public long availableSize() {
		throw new RuntimeException("FileConnection.avilableSize() not implemented.");

	}

	public boolean canRead() {
		return new File(url).canRead();

	}

	public boolean canWrite() {
		return new File(url).canWrite();
	}

	public boolean create() throws IOException {
		FileOutputStream fos = new FileOutputStream(url);
		fos.close();
		return new File(url).exists();
	}

	public boolean delete() {
		return new File(url).delete();
	}

	public long directorySize(boolean includeSubDirs) throws IOException {
		if (!connection.isDirectory()) {
			throw new IOException("FileConnection.directorySize() cannot be invoked on a file: " + url);
		}
		else {
			if (includeSubDirs) {

				// Not yet implemented, but would be no problem.

			}
			else {
				String[] list = connection.list();
				if (list != null) {
					long size = 0L;
					for (int i = 0; i < list.length; i++) {
						File f = new File(url + list[i]);
						if (f.isFile())
							size += f.length();
					}
					return size;
				}
			}
			return 0L;
		}
	}

	public boolean exists() {
		return connection.exists();
	}

	public long fileSize() throws IOException {
		if (connection.isDirectory()) {
			throw new IOException("FileConnection.fileSize() cannot be invoked on a directory: " + url);
		}
		else {
			return connection.length();
		}
	}

	public String getPath() {
		throw new RuntimeException("FileConnection.getPath() not implemented.");
	}

	public String getURL() {
		throw new RuntimeException("FileConnection.getUrl() not implemented.");
	}

	public boolean isDirectory() {
		return new File(url).isDirectory();
	}

	/*
	 * Will be left out. false is returned by default sind jdk1.1.8 does not support isHidden().
	 */
	public boolean isHidden() {
		return false;
	}

	public long lastModified() {
		if (connection.exists()) {
			return connection.lastModified();
		}
		else {
			return 0L;
		}
	}

	public String[] list() {

		if (connection.exists() && connection.isDirectory()) {

			String[] list = connection.list();

			if (list == null)
				return new String[0];

			for (int i = 0; i < list.length; i++) {
				File f = new File(url + list[i]);
				if (f.isDirectory()) {
					list[i] = list[i] + "/";
				}
			}
			return list;
		}
		else {
			return new String[0];
		}
	}

	public String[] list(String filter, boolean includeHidden) throws IOException {
		return list();
	}

	/* This method is used to create a directory that is specified in the 
	 * Connector.open() method. The URL needs to end with a trailing "/" 
	 * in order for a directory to be created.
	 */
	public boolean mkdir() {
		return connection.mkdirs();
	}

	public boolean rename(String newName) {
		if (connection.exists()) {
			return connection.renameTo(new File(newName));
		}
		else {
			return false;
		}
	}

	/*
	 * We be left out ! Currently  RuntimeException is thrown. Perhaps we should simply do nothing.
	 */
	public void setHidden(boolean tf) throws IOException {
		if (connection.isDirectory()) {
			throw new IOException("FileConnection.setHidden() cannot be invoked on a directory: " + url);
		}
		else {
			throw new IOException("FileConnection.setHidden() not implemented.");
		}
	}

	/*
	 * We be left out ! Currently  RuntimeException is thrown. Perhaps we should simply do nothing.
	 */
	public void setReadable(boolean tf) throws IOException {
		if (connection.isDirectory()) {
			throw new IOException("FileConnection.setReadable() cannot be invoked on a directory: " + url);
		}
		else {
			throw new IOException("FileConnection.setReadable() not implemented.");
		}
	}

	/*
	 * We be left out ! Currently  RuntimeException is thrown. Perhaps we should simply do nothing.
	 */
	public void setWriteable(boolean tf) throws IOException {
		if (connection.isDirectory()) {
			throw new IOException("FileConnection.setWritable() cannot be invoked on a directory: " + url);
		}
		else {
			throw new IOException("FileConnection.setWritable() not implemented.");
		}
	}

	/*
	 * We be left out ! Currently  RuntimeException is thrown. Perhaps we should 
	 * return Integer.MAX_VALUE.
	 */
	public long totalSize() {
		throw new RuntimeException("FileConnection.totalSize() not implemented.");
	}

	public void truncate(int byteOffset) throws IOException {

	}

	/*
	 * We be left out ! Currently  RuntimeException is thrown. Perhaps we should 
	 * return Integer.MAX_VALUE or call directorySize(true) on the file system root,
	 */
	public long usedSize() {
		throw new RuntimeException("FileConnection.usedSize() not implemented.");
	}

	public InputStream openInputStream() throws IOException {
		if (connection.isDirectory()) {
			throw new IOException("FileConnection.openInputStream() cannot be invoked on a directory: " + url);
		}
		else {
			return new FileInputStream(url);
		}
	}

	public DataInputStream openDataInputStream() throws IOException {
		if (connection.isDirectory()) {
			throw new IOException("FileConnection.openDataInputStream() cannot be invoked on a directory: " + url);
		}
		else {
			return new DataInputStream(openInputStream());
		}
	}

	public OutputStream openOutputStream() throws IOException {
		if (connection.isDirectory()) {
			throw new IOException("FileConnection.openOutputStream() cannot be invoked on a directory: " + url);
		}
		else {
			return new FileOutputStream(url);
		}
	}

	public DataOutputStream openDataOutputStream() throws IOException {
		if (connection.isDirectory()) {
			throw new IOException("FileConnection.openDataOutputStream() cannot be invoked on a directory: " + url);
		}
		else {
			return new DataOutputStream(openOutputStream());
		}
	}

	public void close() throws IOException {
		connection = null;
		mode = -1;
		url = null;
	}
}

/*
 * $Log:$
 */