package org.me4se.impl.lcdui;

import java.awt.*;
import java.util.*;
import javax.microedition.midlet.*;

public class FontInfo {

    public static final int LEFT = -1;
    public static final int RIGHT = 1;
    public static final int CENTER = 0;

    public static final int BORDER = 2;
    public static final int SHADOW = 4;
    public static final int ROUND = 8;
    public static final int UNDERLINE = 16;
    public static final int OVERLINE = 32;
    public static final int LIGHT = 64;

    String type;
    public PhysicalFont font;
    
    public boolean underline;

    public Color foreground;
    public Color background;

    public int align;
    public int decoration;
    
    /** 
     * height of the frame/ui element. The font height is stored with the 
     * physical font */
    
    public int height; 

    public static Hashtable infoCache = new Hashtable();
    public static Hashtable physicalCache = new Hashtable ();

    /** type string as used in the property file, e.g. "SoftButton", 
     * 	"system.bold.medium" */

    public static FontInfo getFontInfo(String type) {

        FontInfo result = (FontInfo) infoCache.get(type);
        if (result == null) {
            result = new FontInfo (type);
            infoCache.put(type, result);
        }
        return result;
    }


    protected FontInfo(String type) {

        this.type = type;
        ApplicationManager manager = ApplicationManager.manager;

        // handle color
        if (type.equals("label"))
            underline = true;

        height = manager.getIntProperty(type+".height", -1);
        int bg = manager.getIntProperty(type + ".background", -1);
        int fg = manager.getIntProperty(type + ".foreground", -1);

        if (bg == -1 && type.endsWith(".focus"))
            bg = manager.getIntProperty("default.focus.background", -1);

        if (fg == -1 && type.endsWith(".focus"))
            fg = manager.getIntProperty("default.focus.foreground", -1);

        boolean customColor = bg != -1 || fg != -1;

        if (bg == -1)
            bg = type.equals("softButton") ? 0 : 0x0ffffff;
        if (fg == -1)
            fg = type.equals("softButton") ? 0x0ffffff : 0;

        background = new Color(manager.getDeviceColor(bg, false));
        foreground = new Color(manager.getDeviceColor(fg, false));

        // handle alignment

        String alignS = manager.properties.getProperty(type + ".align");

        if (alignS == null)
            align = type.equals("softButton") ? CENTER : LEFT;
        else if ("right".equals(alignS))
            align = RIGHT;
        else if ("center".equals(alignS))
            align = CENTER;
        else if ("left".equals(alignS))
            align = LEFT;
        else if ("border".equals(alignS))
            align = BORDER;

        // handle decoration

        String decorationStr =
            manager.properties.getProperty(type + ".decoration");

        if (decorationStr == null) {
            if (type.equals("item.focus") && !customColor)
                decoration = SHADOW;
            else if (type.equals("title"))
                decoration = UNDERLINE;
        }
        else {
            if (decorationStr.indexOf("shadow") != -1)
                decoration |= SHADOW;
            if (decorationStr.indexOf("underline") != -1)
                decoration |= UNDERLINE;
            if (decorationStr.indexOf("light") != -1)
                decoration |= LIGHT;
            if (decorationStr.indexOf("overline") != -1)
                decoration |= OVERLINE;
            if (decorationStr.indexOf("decoration") != -1)
                decoration |= BORDER;
        }

        // ok, look for the physical font
        
        String description = (String) manager.properties.get("font."+type);
        if (description == null && type.endsWith (".focus"))
            description = (String) manager.properties.get("font.focus");
        if (description == null) 
            description = (String) manager.properties.get("font.default");
        
        if (description != null) {
            font = (PhysicalFont) physicalCache.get (description);
            if (font == null) {
                font = description.endsWith (".properties") 
		    ? getAwtFont () 
		    : new AwtFont (description);
                physicalCache.put(description, font);   
            }
        }
        else {
            font = getAwtFont ();
        }

        if (height == -1) height = font.height;
        
        //stem.out.println ("requested: "+type+" got size: "+size);
    }

    public AwtFont getAwtFont() {

        boolean bold = ((type.equals("softButton")
                || "title".equals(type)
                || type.indexOf(".bold") != -1
                || type.indexOf(".focus") != -1));

        boolean italic = type.indexOf(".italic") != -1;

        int height;

        if (font != null)
            height = font.height;
        else {
            height = 14;

            if (type.indexOf(".small") != -1)
                height = height * 3 / 4;
            else if (type.indexOf(".large") != -1)
                height = height * 3 / 2;
        }

        return new AwtFont (height, bold, italic);
    }

    public void drawString(Graphics g, String s, int x, int y) {
        font.drawString (g, s, x, y);
              
        if (underline)
            g.drawLine(x, y + 1, x + font.stringWidth(s), y + 1);
    }
}
