package org.me4se.impl.lcdui;

import org.kobjects.scm.ScmComponent;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.util.Vector;
import javax.microedition.lcdui.TextField;

/**
 * @author Stefan Haustein
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 */
public class TextComponent extends DeviceComponent {

	Vector lines = new Vector();
	int y0;
	int cx;
	int cy;
	int constraints;

	public TextComponent(String type) {
		super(type, true);
		lines.addElement("");
	}

	boolean check(char c) {
		switch (constraints & TextField.CONSTRAINT_MASK) {
			case TextField.PHONENUMBER :
				if ("/()#* ".indexOf(c) != -1)
					return true;
			case TextField.NUMERIC :
				return (c >= '0' && c <= '9');
			default :
				return true;
		}
	}

	String getLine(int n) {
		return (String) lines.elementAt(n);
	}

	public void paint(Graphics g) {
		super.paint(g);

		FontInfo fi = getFontInfo();
		PhysicalFont pf = fi.font;

		g.setColor(fi.foreground);


		if (cy >= lines.size())
			cy = lines.size() - 1;

		if (cy*pf.height-y0 >= getHeight ())
		 y0 = cy*pf.height;
		else if (y0 > cy*pf.height)
		 y0 = cy*pf.height;

		int y = -y0;

		for (int i = 0; i < lines.size(); i++) {

			String line = getLine(i);
			fi.drawString(g, line, 0, y + pf.ascent);

			if (i == cy && hasFocus()) {
				if (cx > line.length())
					cx = line.length();
				int x = pf.stringWidth(line.substring(0, cx));
				g.fillRect(x, y, 2, pf.height);
			}

			y += pf.height;
		}
	}

	public boolean keyPressed(int code, int mode) {
		if (code == KeyEvent.VK_LEFT) {
			if (cx > 0)
				cx--;
			else if (cy > 0) {
				cy--;
				cx = getLine(cy).length();
			}
			repaint();
			return true;
		}
		else if (code == KeyEvent.VK_RIGHT) {
			if (cx < getLine(cy).length())
				cx++;
			else if (cy < lines.size() - 1) {
				cy++;
				cx = 0;
			}
			repaint();
			return true;
		}

		return super.keyPressed(code, mode);
	}

	public boolean keyTyped(char c) {
		String line = getLine(cy);

		if (c == 8) {
			if (cx == 0 && cy > 0) {
				line = getLine(--cy);
				cx = line.length();
			}

			if (cx > 0) {
				line = line.substring(0, cx - 1) + line.substring(cx);
				cx--;
			}

			lines.setElementAt(line, cy);
			doLayout();
		}
		else if (c >= 32 && check(c)) {
			line = line.substring(0, cx) + c + line.substring(cx);
			cx++;

			lines.setElementAt(line, cy);
			if (getFontInfo().font.stringWidth(line) > getWidth())
				doLayout();
		}

		
		repaint();
		return true;
	}

	public int getConstraints() {
		return constraints;
	}

	public String getText() {
		StringBuffer buf = new StringBuffer();

		for (int i = 0; i < lines.size(); i++)
			buf.append((String) lines.elementAt(i));

		return buf.toString();
	}

	public int getCharPos() {
		int pos = cx;
		for (int i = 0; i < cy; i++)
			pos += getLine(i).length();
		return pos;
	}

	public void doLayout() {
//		System.out.println("relayouting");

		int charPos = getCharPos();
		String text = getText();
		lines = new Vector();
//		System.out.println ("full text: "+text);

		WordWrap ww = new WordWrap(getFontInfo().font, text, getWidth());
		int p0 = 0;
		cx = 0;
		cy = 0;
		while (true) {
			int p1 = ww.next();
			if (p1 == -1)
				break;
			int chars = p1 - p0;
			if (charPos > chars) {
				charPos -= chars;
				cy++;
			}
			else if (charPos > 0) {
				cx = charPos;
				charPos = 0;
			}
			String l = text.substring (p0, p1);
//			System.out.println ("p0: "+p0+" p1:"+p1+" appending line: "+l);
			lines.addElement(l);
			p0 = p1;
		}

		if (lines.size() == 0 || charPos > 0)
			lines.addElement("");

	}

	public void setConstraints(int constraints) {
		this.constraints = constraints;
	}

	public void setText(String text) {
		lines = new Vector();
		lines.addElement(text);
		invalidate();
	}

	public Dimension getMinimumSize() {

		return new Dimension(20, getFontInfo().height);
	}

}
